@@ -6,4 +6,7 @@ if dirName not in sys.path:
 
 from solvers import *
 from utilities import *
-from analysis import *
\ No newline at end of file
+from analysis import *
+from fileio import *
+
+import solvers, utilities, analysis, fileio
\ No newline at end of file
