@@ -1,7 +1,5 @@
 from fileio import *
 
-#Appears to be common/best practice to import required packages in every file
-#they are used in
 import numpy as np
 #import numdifftools as nd
 import numdifftools as nd
@@ -1064,7 +1062,8 @@ class SurfaceUtils:
         return indsOut, gridValsOut
     
     @staticmethod
-    def find_endpoints_on_grid(coordMeshTuple,potArr,returnAllPoints=False,eneg=0):
+    def find_endpoints_on_grid(coordMeshTuple,potArr,returnAllPoints=False,eneg=0,
+                               returnIndices=False):
         """
         
 
@@ -1077,7 +1076,7 @@ class SurfaceUtils:
         -------
         allowedEndpoints : TYPE
             DESCRIPTION.
-        allowedIndices : asdf
+        allowedIndices : TYPE
 
         """
         if returnAllPoints:
@@ -1087,6 +1086,8 @@ class SurfaceUtils:
         nDims = len(coordMeshTuple)
         uniqueCoords = [np.unique(c) for c in coordMeshTuple]
         
+        potArr = _get_correct_shape(uniqueCoords,potArr)
+        
         allContours = SurfaceUtils.find_approximate_contours(coordMeshTuple,potArr,eneg=eneg)
         
         allowedEndpoints = np.zeros((0,nDims))
@@ -1116,8 +1117,11 @@ class SurfaceUtils:
         
         allowedEndpoints = np.unique(allowedEndpoints,axis=0)
         allowedIndices = np.unique(allowedIndices,axis=0)
-                
-        return allowedEndpoints, allowedIndices
+        
+        if returnIndices:
+            return allowedEndpoints, allowedIndices
+        else:
+            return allowedEndpoints
 
 def shift_func(func_in,shift=10**(-4)):
     """
@@ -1142,6 +1146,46 @@ def shift_func(func_in,shift=10**(-4)):
         return func_in(coords) - shift
     return func_out
 
+def _get_correct_shape(gridPoints,arrToCheck):
+    """
+    Utility for automatically correcting the shape of an array, to deal with
+    nonsense regarding np.meshgrid's default setup
+
+    Parameters
+    ----------
+    gridPoints : TYPE
+        DESCRIPTION.
+    arrToCheck : TYPE
+        DESCRIPTION.
+
+    Returns
+    -------
+    None.
+
+    """
+    defaultMeshgridShape = np.array([len(g) for g in gridPoints])
+    possibleOtherShape = tuple(defaultMeshgridShape)
+    defaultMeshgridShape[[1,0]] = defaultMeshgridShape[[0,1]]
+    defaultMeshgridShape = tuple(defaultMeshgridShape)
+    
+    isSquare = False
+    if defaultMeshgridShape[0] == defaultMeshgridShape[1]:
+        isSquare = True
+        warnings.warn("Grid is square; cannot check if data is transposed."+\
+                      " Note that gridVals should be of shape (x.size,y.size).")
+    
+    if arrToCheck.shape == defaultMeshgridShape:
+        pass
+    elif arrToCheck.shape == possibleOtherShape and not isSquare:
+        arrToCheck = np.swapaxes(arrToCheck,0,1)
+    else:
+        raise ValueError("arrToCheck.shape "+str(arrToCheck.shape)+\
+                         " does not match expected shape "+\
+                         str(defaultMeshgridShape)+" or possible shape "+\
+                         str(possibleOtherShape))
+    
+    return arrToCheck
+
 class NDInterpWithBoundary:
     """
     Interpolates a grid in D dimensions, with extra handling for points outside
@@ -1150,7 +1194,7 @@ class NDInterpWithBoundary:
     :Maintainer: Daniel
     """
     def __init__(self,gridPoints,gridVals,boundaryHandler="exponential",symmExtend=None,\
-                 splKWargs={}):
+                 transformFuncName="identity",splKWargs={}):
         """
         Initializes the class instance. Carries out basic error checking on inputs.
         Defines self._call as the method to evaluate a point that's within the
@@ -1170,6 +1214,9 @@ class NDInterpWithBoundary:
         symmExtend : bool or ndarray of bools, optional
             Whether to symmetrically extend gridVals when evaluating. See notes.
             The default is None.
+        transformFuncName : string, optional
+            The function to apply to the interpolated function after interpolating.
+            The default is "identity", in which no post-processing is applied.
         splKWargs : dict, optional
             Extra arguments for spline interpolation, in the 2D case. The default
             is {}.
@@ -1220,22 +1267,8 @@ class NDInterpWithBoundary:
         
         self.symmExtend = symmExtend
         
-        defaultMeshgridShape = tuple([len(g) for g in gridPoints])
-        possibleOtherShape = np.array(defaultMeshgridShape)
-        possibleOtherShape[[1,0]] = possibleOtherShape[[0,1]]
-        possibleOtherShape = tuple(possibleOtherShape)
-        
-        if defaultMeshgridShape[0] == defaultMeshgridShape[1]:
-            warnings.warn("Grid is square; cannot check if data is transposed."+\
-                          " Note that gridVals should be of shape (x.size,y.size).")
-        
-        if gridVals.shape != defaultMeshgridShape:
-            if gridVals.shape == possibleOtherShape:
-                gridVals = np.swapaxes(gridVals,0,1)
-            else:
-                raise ValueError("gridVals.shape does not match expected shape "+\
-                                 str(defaultMeshgridShape)+" or possible shape "+\
-                                 str(possibleOtherShape))
+        self.gridPoints = tuple([np.asarray(p) for p in gridPoints])
+        self.gridVals = _get_correct_shape(gridPoints,gridVals)
         
         for i, p in enumerate(gridPoints):
             if not np.all(np.diff(p) > 0.):
@@ -1243,13 +1276,14 @@ class NDInterpWithBoundary:
                                  "ascending" % i)
         
         if self.nDims == 2:
-            self.rbv = RectBivariateSpline(*gridPoints,gridVals,**splKWargs)
+            self.rbv = RectBivariateSpline(*gridPoints,self.gridVals.T,**splKWargs)
             self._call = self._call_2d
         else:
             self._call = self._call_nd
-                
-        self.gridPoints = tuple([np.asarray(p) for p in gridPoints])
-        self.gridVals = gridVals
+            
+        postEvalDict = {"identity":self._identity_transform_function,
+                        "smooth_abs":self._smooth_abs_transform_function}
+        self.post_eval = postEvalDict[transformFuncName]
 
     def __call__(self,points):
         """
@@ -1296,7 +1330,8 @@ class NDInterpWithBoundary:
             else:
                 result[ptIter] = self.boundaryHandler(point,isInBounds)
         
-        result = result.reshape(originalShape) 
+        result = self.post_eval(result)
+        result = result.reshape(originalShape)
         
         return result
     
@@ -1470,6 +1505,28 @@ class NDInterpWithBoundary:
         result = valAtNearest*np.exp(np.sqrt(dist))
         return result
     
+    def _identity_transform_function(self,normalEvaluation):
+        """
+        Not sure if it's faster to have this dummy function in place, or to have
+        an "if-else" statement every time we check if we should call a transform
+        function.
+
+        Parameters
+        ----------
+        normalEvaluation : TYPE
+            DESCRIPTION.
+
+        Returns
+        -------
+        normalEvaluation : TYPE
+            DESCRIPTION.
+
+        """
+        return normalEvaluation
+    
+    def _smooth_abs_transform_function(self,normalEvaluation):
+        return np.sqrt(normalEvaluation**2 + 10**(-4))
+    
 class PositiveSemidefInterpolator:
     def __init__(self,gridPoints,listOfVals,ndInterpKWargs={}):
         """
@@ -1477,13 +1534,18 @@ class PositiveSemidefInterpolator:
 
         Parameters
         ----------
-        gridPoints : TYPE
-            DESCRIPTION.
-        listOfVals : TYPE
-            Expected to be ordered by row for the matrix, e.g.
-                [[l[0],l[1],...,l[n]],
-                 [-,l[n+1],...,l[2n-1]],
-                 ...]
+        gridPoints : tuple
+            Elements are the unique coordinates of the grid. Shapes are
+                (N1,N2,...,Nn).
+        listOfVals : list
+            A positive semidefinite matrix M has unique values
+                M = [[M00, M01, ..., M0n],
+                     [M01, M11, ..., M1n],
+                     ...,
+                     [M0n, M1n, ..., Mnn]].
+            The components of listOfVals are the numpy arrays
+                [M00, M01, ..., M0n, M11, M12, ..., M1n, ..., Mnn].
+            Each Mij is of shape (N2,N1,N3,...), as in the output of np.meshgrid.
         ndInterpKWargs : TYPE, optional
             DESCRIPTION. The default is {}.
 
@@ -1497,40 +1559,22 @@ class PositiveSemidefInterpolator:
         
         #Stupid case for nDims == 1. For higher dimensions, pass through
         #NDInterpWithBoundary in components individually
-        if self.nDims != 2: #Unused case self.nDims == 1
+        if self.nDims != 2:
             raise NotImplementedError
         
         #Standard error checking
-        assert self.nDims == len(listOfVals)
+        assert len(listOfVals) == int(self.nDims*(self.nDims+1)/2)
         
         for i, p in enumerate(gridPoints):
             if not np.all(np.diff(p) > 0.):
                 raise ValueError("The points in dimension %d must be strictly "
                                  "ascending" % i)
         
-        defaultMeshgridShape = np.array([len(g) for g in gridPoints])
-        possibleOtherShape = tuple(defaultMeshgridShape)
-        defaultMeshgridShape[[1,0]] = defaultMeshgridShape[[0,1]]
-        defaultMeshgridShape = tuple(defaultMeshgridShape)
+        self.gridValsList = [_get_correct_shape(gridPoints,l) for l in listOfVals]
         
-        if defaultMeshgridShape[0] == defaultMeshgridShape[1]:
-            warnings.warn("Grid is square; cannot check if data is transposed."+\
-                          " Note that gridVals should be of shape (x.size,y.size).")
-        
-        self.gridValsList = []
-        for l in listOfVals:            
-            if l.shape == defaultMeshgridShape:
-                self.gridValsList.append(l)
-            elif l.shape == possibleOtherShape:
-                self.gridValsList.append(np.swapaxes(l,0,1))
-            else:
-                raise ValueError("One supplied gridVals.shape does not match expected shape "+\
-                                 str(defaultMeshgridShape)+" or possible shape "+\
-                                 str(possibleOtherShape))
-                    
         #Taking shortcuts because I only care about D=2 right now
-        self.gridVals = np.stack((np.stack(self.gridValsList[0],self.gridValsList[1]),\
-                                  np.stack(self.gridValsList[1],self.gridValsList[2])))
+        self.gridVals = np.stack((np.stack((self.gridValsList[0],self.gridValsList[1])),\
+                                  np.stack((self.gridValsList[1],self.gridValsList[2]))))
         self.gridVals = np.moveaxis(self.gridVals,[0,1],[2,3])
         
         self.eigenVals, self.eigenVecs = np.linalg.eig(self.gridVals)
@@ -1538,22 +1582,39 @@ class PositiveSemidefInterpolator:
         
         #Constructing interpolators
         self.eigenValInterps = [NDInterpWithBoundary(self.gridPoints,e,**ndInterpKWargs)\
-                                for e in self.eigenVals]
+                                for e in self.eigenVals.T]
         self.eigenVecInterp = NDInterpWithBoundary(self.gridPoints,thetaVals,**ndInterpKWargs)
         
     def __call__(self,points):
+        originalShape = points.shape[:-1]
+        if originalShape == ():
+            originalShape = (1,)
+        
+        if points.shape[-1] != self.nDims:
+            raise ValueError("The requested sample points have dimension "
+                             "%d, but this NDInterpWithBoundary expects "
+                             "dimension %d" % (points.shape[-1], self.nDims))
+            
+        points = points.reshape((-1,self.nDims))
+        
         eigenVals = [e(points) for e in self.eigenValInterps]
         theta = self.eigenVecInterp(points)
         
+        ct = np.cos(theta)
+        st = np.sin(theta)
+        
         eigenVals = [e.clip(0) for e in eigenVals]
         
-        ret = np.zeros(points.shape[:-1]+(self.nDims,self.nDims))
-        # evalMat = np.stack([np.diag(e) for e in np.stack(eigenVals).T])
-        # evecMat = 
-        
-        return None
-                    
+        ret = np.zeros((len(points),2,2))
+        for (ptIter,point) in enumerate(points):
+            ret[ptIter,0,0] = eigenVals[0][ptIter]*ct[ptIter]**2 + \
+                eigenVals[1][ptIter]*st[ptIter]**2
+            ret[ptIter,1,0] = (eigenVals[1][ptIter]-eigenVals[0][ptIter])*st[ptIter]*ct[ptIter]
+            ret[ptIter,0,1] = ret[ptIter,1,0]
+            ret[ptIter,1,1] = eigenVals[0][ptIter]*st[ptIter]**2 + \
+                eigenVals[1][ptIter]*ct[ptIter]**2
                 
+        return ret.reshape(originalShape+(2,2))
     
 def mass_funcs_to_array_func(dictOfFuncs,uniqueKeys):
     """
